module coarsen3
! Coarsen mesh.
  use adj_mod
  use helixcartmod
! Model mm(m1,m2) is coarse.  Data dd(n1,n2) linear interpolated.
!
!  constructs an xy array of the output point locations
!  Operator then calls the linear interpolation operator to
!  map the smaller gridded data to the expanded (in the sense
!  of area) ouput.
! 
!
!
  use lint3
  implicit none
  real,   dimension( :, :), pointer :: xyp 
  contains
  subroutine coarsen3_init ( fo,fd,fn,co,cd,cn)
    real, pointer, dimension (:)    :: co, cd
    integer, pointer, dimension (:) :: cn, fn
    real, pointer, dimension (:)    :: fo, fd
    integer                         :: in, ic, id, ie, dim, nd
    real, dimension(3)               :: nmin, nmax, loc_fine
    integer, dimension(3)            :: iloc_coarse, iloc, iloc_fine

    dim = size(fo)
    nd = product(fn)
    allocate (xyp(nd,3),co(3),cd(3))

    nmin = fo-fd/2
    nmax = fo +fd*(fn-1) +fd/2        ! Great formula!
    cd = (nmax-nmin)/cn
    co = (nmin*(cn-.5) + nmax/2)/cn    ! Great formula!

   !loop through the fine space
   do ic=1,nd 
     call helix2cart(fn,ic,iloc_fine) ! my current index in fine space
     loc_fine= (iloc_fine-1.)*fd+fo ! my current position in fine space
     xyp(ic,:) = loc_fine
   end do 
    call lint3_init( cn, co, cd, xyp)
  end subroutine
  function coarsen3_lop ( adj, add, mm, dd) result(stat)
    integer            :: stat 
    logical,intent(in) :: adj,add 
    real,dimension(:)  :: mm,dd 
!write(0,*)"coarsen3lop crap1"
    call adjnull (adj,add,mm,dd )
!write(0,*)"coarsen3lop crap2"
    call coarsen3_lop2(adj,mm,dd )
    stat=0
  end function 
  subroutine coarsen3_lop2(adj,mm,dd)
    logical,intent(in) :: adj
    real, dimension (:)  :: mm 
    real, dimension (:)  :: dd 
    integer stat1
!write(0,*)"coarsen3lop crap3",size(mm),minval(mm),maxval(mm),size(dd),minval(dd),maxval(dd)
    stat1 = lint3_lop( adj, .true., mm, dd)
!write(0,*)"coarsen3lop crap4"
  end subroutine
  subroutine  coarsen3_close()
    deallocate (xyp)
    call lint3_close()
  end subroutine 
end module 














