module coarsen3_norm

use sep
use adj_mod
use chain_mod
use coarsen3
use weight

implicit none

integer, private, save     :: n1, n2
real, private, save        :: d1, d2, o1, o2
real, dimension(:), pointer :: fold

contains

! ---------------------------------------------------------------------------
  subroutine coarsen3_norm_init( fo,fd,n,co,cd,m,maskin,normlz )
    optional              ::                                maskin,normlz
    integer               :: stat
    integer, dimension(:), pointer :: m,n
    real, pointer         :: co(:),cd(:),maskin(:), mask(:),fo(:),fd(:)           ! known mask on fine grid
    logical               :: norm, normlz                   ! Normalize or not?
    
    ! Handle the optional arguments.
    norm = .true.
    if( present(normlz) ) norm=normlz
    if( .not. present(maskin) ) then
       allocate( mask(product(n)) )
       mask=1.0                                     ! data space vector of ones
    else
       mask => maskin
    end if
    
    ! determine fold by refining mask
    allocate( fold(product(m)) )
    if( norm) then
       call coarsen3_init( fo, fd, n, co, cd, m)
       stat = coarsen3_lop( .true., .false., fold, mask )
       call coarsen3_close()     
    else
       fold = 1.0
    end if
    where( fold/=0. ) fold=1./fold
    call weight_init(fold)       
    if( .not. present(maskin) )  deallocate( mask )
    call coarsen3_init( fo, fd, n, co, cd, m)
    
  end subroutine coarsen3_norm_init
! ---------------------------------------------------------------------------
        
! ---------------------------------------------------------------------------
        
  function coarsen3_norm_lop ( adj, add, mm, dd) result(stat)
    integer            :: stat 
    logical,intent(in) :: adj,add 
    real,dimension(:)  :: mm,dd 
    call adjnull (adj,add,mm,dd )
    call coarsen3_norm_lop2(adj,add,mm,dd )
    stat=0
  end function 

  subroutine coarsen3_norm_lop2( adj,add, mm, dd)
    logical            :: adj,add
    real, dimension(:) :: mm,dd,temp 

    allocate( temp(size(mm)) )

! chain refine with weight for normalization
    call chain( coarsen3_lop, weight_lop, adj,add, mm, dd, temp )
    deallocate(temp)
    
  end subroutine coarsen3_norm_lop2
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
  subroutine coarsen3_norm_close( )
    deallocate( fold)
    call coarsen3_close()
  end subroutine coarsen3_norm_close
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
  subroutine get_fold( foldout )
    real, dimension(:), pointer :: foldout
    foldout = fold
  end subroutine get_fold
! ---------------------------------------------------------------------------


end module coarsen3_norm

