module nmshconest3

use nhelix
use nhconest
use sep
use adj_mod
use coarsen3_norm
use nbound_bob_mod        
use regridn        
use nmisinput

implicit none

type(nfilter), private, save         :: aa
integer, private, save               :: dim, ns, stat
real, dimension(:), pointer, save    :: data0, knownmask, dfg, ofg, xmin, xmax, databank, maskbank, patchbank
integer, dimension(:), pointer, save :: sc2, oldpatch,  nfg, nag, nsg, mark, lagbank, lagmark, nbank
logical, dimension(:), pointer, save :: k0bank, misbank

contains

  subroutine nmshconest3_init( data00, aa0, sc0, nf, df, of, na, k00 )
    type(nfilter)                    :: aa0
    real, dimension(:), pointer      :: data00, df, of, rd, ro
    integer, dimension(:), pointer   :: sc0, nf, na, newcart, rn, patch, oldcart
    integer                          :: datsize, is, ip, banksize, oldline, stat, neq, nlag
    real, dimension(:), pointer      :: k00

write(0,*)"size1 init"
    dim=size(nf)
write(0,*)"size2 init"
    ns = size(sc0)
write(0,*)"size3 init"
    allocate( data0(size(data00)), knownmask(size(k00)),oldpatch(size(aa0%pch)), sc2(ns), mark(ns+1), lagmark(ns+1))
    allocate( xmin(dim),xmax(dim), dfg(dim), ofg(dim), nfg(dim), nag(dim), nsg(dim),newcart(dim),rn(dim),oldcart(dim))
    aa = aa0; data0 = data00; knownmask = k00
    sc2 = sc0; dfg = df; ofg = of; nfg = nf; nag = na
    xmin = ofg-dfg/2
    xmax = ofg +dfg*(nfg-1) + dfg/2
    oldpatch = aa%pch
    mark(1)=1; lagmark(1)=1
    do is=2,ns+1
       mark(is) = mark(is-1) + product(nint((real(sc2(is-1))/real(nfg(1)))*real(nfg)))
write(0,*)"size4 init"
       lagmark(is) = lagmark(is-1) + size(aa%hlx(1)%lag)
    end do
    banksize = mark(ns+1)-1
    allocate(databank(banksize),maskbank(banksize),patchbank(banksize),k0bank(banksize),misbank(banksize))
write(0,*)"size5 init"
    allocate(lagbank(size(aa%hlx(1)%lag)*ns),nbank(dim*ns))
    k0bank = .false.
    lagbank = 0

    if (allocated(aa%mis)) then 
       deallocate(aa%mis)
    endif
    if (allocated(aa%pch)) then 
       deallocate(aa%pch)
    endif


! Loop over scales, set up everything
    do is=1,ns
       do ip=1,dim  !The dimensions for scale is
          rn(ip) = nint((real(sc2(is))/real(nfg(1)))*real(nfg(ip)))
          nbank((is-1)*dim+ip)=rn(ip)
       end do
       allocate(patch(product(rn)))
       call coarsen3_norm_init(ofg, dfg, nfg, ro, rd, rn, knownmask)
       stat = coarsen3_norm_lop( .true., .false., databank(mark(is):(mark(is+1)-1)), data0)
       call coarsen3_norm_close()
       call coarsen3_norm_init(ofg, dfg, nfg, ro, rd, rn, knownmask)
       stat = coarsen3_norm_lop( .true., .false., maskbank(mark(is):(mark(is+1)-1)), knownmask)	
       call coarsen3_norm_close()
       do ip=1,product(rn)
         call line2cart(rn,ip,newcart)
         oldcart = newcart*nfg/rn
         call cart2line(nfg,oldcart,oldline)
         patch(ip) = oldpatch(oldline)
       end do
       patchbank(mark(is):mark(is+1)-1) = patch
       deallocate(patch)
       where (maskbank(mark(is):mark(is+1)-1)==0.)
          k0bank(mark(is):mark(is+1)-1)=.true.
       end where
       allocate(aa%mis(product(rn)),aa%pch(product(rn)))
       aa%mis=.false.
       aa%pch=patchbank(mark(is):mark(is+1)-1)
       call nregridn(nfg,rn,aa) 
       call nbound_bob(rn,aa,k0bank(mark(is):mark(is+1)-1))
       misbank(mark(is):mark(is+1)-1) = aa%mis
                                                               write(0,*)"is, count, size, mark",is,size(aa%mis)-count(aa%mis),size(aa%mis)
       lagbank(lagmark(is):lagmark(is+1)-1)=aa%hlx(1)%lag

       deallocate(aa%mis)
       call nregridn(rn,nfg,aa)
    end do
       deallocate(maskbank)
       neq = 0
       do ip=1,banksize
          if (k0bank(ip)) then
          neq=neq+1
          endif
       end do
     write(0,*)"Number of fitting equations:",neq
  end subroutine nmshconest3_init
! ---------------------------------------------------------------------------
! ---------------------------------------------------------------------------    
       
  function nmshconest3_lop ( adj, add, mm, dd) result(stat)
    integer            :: stat 
    logical,intent(in) :: adj,add 
    real,dimension(:)  :: mm,dd 
    call adjnull (adj,add,mm,dd )
    call nmshconest3_lop2(adj,mm,dd )
    stat=0
  end function 

  subroutine nmshconest3_lop2( adj, mm, dd)
    logical                        :: adj 
    real, dimension(:)             :: mm, dd
    real, dimension(:), pointer    :: datatmp, datar
    integer, dimension(dim)        :: rn
    integer                        :: is, datsize, ia

    if (allocated(aa%pch)) then
       deallocate(aa%pch)
    endif
    if (allocated(aa%mis)) then
       deallocate(aa%mis)
    endif
    do is=1,ns                                      ! looping over scales
       rn = nbank((is-1)*dim+1:is*dim)
       datsize = product(rn)
       allocate(aa%mis(datsize),aa%pch(datsize),datatmp(datsize),datar(datsize))
       datatmp = 0.0
       aa%mis = misbank(mark(is):mark(is+1)-1)
       aa%pch = patchbank(mark(is):mark(is+1)-1)
       do ia=1,maxval(aa%pch)
          aa%hlx(ia)%lag=lagbank(lagmark(is):lagmark(is+1)-1)
       end do
       datar = databank(mark(is):mark(is+1)-1)
       call nhconest_init( datar, aa,size(aa%hlx(1)%flt) )
       datatmp = dd(mark(is):mark(is+1)-1)
       stat = nhconest_lop( adj, .true., mm, datatmp )  ! Just normal nhconest--note add=.true.
       dd(mark(is):mark(is+1)-1) = datatmp
       deallocate( datatmp, datar)
       deallocate(aa%pch,aa%mis)
    end do
  end subroutine nmshconest3_lop2
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
  subroutine nmshconest3_close( )
		deallocate(data0,knownmask,sc2,dfg,ofg,nfg,nag,xmin,xmax,oldpatch,databank,k0bank,patchbank,misbank,lagbank)
  end subroutine nmshconest3_close
! ---------------------------------------------------------------------------

   end module nmshconest3







