module nmultipefe3
! Find multi-scale prediction-error filter (helix magic)
  use nmshconest3
  use cgstep_mod
!  use solver_mod
  use solver_reg_mod
  use coarsen3_norm
  use laplacf3
  use sep
  use irls
  implicit none
  contains

  subroutine nmultipef3_find( yy, aa, sc1, on, od, oo, a, niter, k0, patch, eps)
  ! yy=data aa=filter sc1=scales on,od,oo=datadim a=filtersize niter=#iterations k0=mask

    integer,             intent( in)              :: niter
    integer, dimension(:),pointer                 :: sc1, on, a, cn, patch, oldtable, oldlag
    real, dimension(:), pointer                   :: yy, k0, ddr1, ee, temp, od, oo, nd, no,xming,xmaxg,cd,co
    type(nfilter)                                 :: aa
    integer                                       :: is, stat, dim, id, sf, sp, naops
    real                                          :: df, eps, scsize
    real                                          :: oo1d, oo2d, od1d, od2d
    integer, dimension(:), allocatable            :: mark

  ! set up positions in ee for subsampled data (right side of equation (2) ) 
    dim = size(on)
    allocate (mark(size(sc1)+1))
    mark(1) = 1
    do is = 1, size(sc1)
      scsize = product(on)*((real(sc1(is))/real(on(1)))**dim)
      mark(is+1) = mark(is)+nint(scsize)
    end do

    sf = size(aa%hlx(1)%flt)
    sp = size(aa%hlx)

  ! allocate ee
    allocate( cn(dim), xming(dim), xmaxg(dim), cd(dim), co(dim) )
    allocate( ee( mark(size(sc1)+1)-1 ), temp(sf*sp), oldtable(product(on)), oldlag(sf) )

  ! fill ee with various scales of data
    do is = 1, size(sc1)      
        cn = nint((real(sc1(is))/real(on(1)))*real(on))
        allocate( ddr1(product(cn)) );  ddr1=0  
        call coarsen3_norm_init( oo, od, on, co, cd, cn, k0)  
        stat = coarsen3_norm_lop( .true., .false., ddr1, yy)   ! Subsamp data to iscale
                                                                write(0,*)"coarsen min max size rmask",is,minval(ddr1),maxval(ddr1),size(ddr1)
        ee(mark(is):mark(is+1)-1) = ddr1
        deallocate(ddr1)
        call coarsen3_norm_close()
    end do 

    oldtable=aa%pch
    oldlag = aa%hlx(1)%lag
    aa%pch=oldtable
    naops=product(patch)*size(aa%hlx(1)%flt)
    temp=0.

    call nmshconest3_init( yy, aa, sc1, on, od, oo, a, k0)
    deallocate(aa%pch)
    call laplacf3_init(patch(1), patch(2), patch(3),size(aa%hlx(1)%flt))
    call solver_reg( m=temp, d=-ee, Fop=nmshconest3_lop, Aop=laplacf3_lop, &
stepper=cgstep, nAop=naops, niter=niter, eps=eps, verb=.true.)

!    call solver_reg(oper=nmshconest3_lop, solv=cgstep, reg=laplacf3_lop, &
!nreg=naops, x=temp, dat=-ee, niter=niter, eps=eps, verb=.true.)

!    call solver_reg(oper=nmshconest3_lop, solv=cgstep, reg=laplacf3_lop, &
!nreg=naops, x=temp, dat=-ee, niter=niter, eps=eps, wght=cauchy, verb=.true.)

    call cgstep_close() 
    call nmshconest3_close( )

    allocate(aa%pch(product(on)))
    aa%pch = oldtable
    do is=1,sp
       aa%hlx(is)%lag=oldlag
       aa%hlx(is)%flt = temp((1+(is-1)*sf):(1+(is)*sf))
    end do
    deallocate(ee,mark,temp,oldtable)
  end subroutine 
end module 













