Program NMultipef3
! Three dimensional, non-stationary, multi-scale PEF-based interpolator
! niter - number of iterations to find NSMSPEF
! rad - radial or laplacian smoothing of micropatches (not yet implemented)
! eps - value of epsilon for NSMSPEF regularization
! filliter - number of iterations to fill data with NSMSPEF
! maskin - mask of known data (ones where data is known, zeroes where data is missing)
! a - size of PEF, center - position of leading 1 in PEF, gap - size of zeroes after 1
! patch - number of patches in each dimension (should divide cleanly in all dimensions)
! sc - scales to use, based upon the first dimension.  Must include original scale

use sep
use nhelix
use createnhelixmod
use nbound_bob_mod
use nmisinput
use compress
use print
use nmultipefe3
use pef
use io_nfilt
!use nsfill

  implicit none
  integer                              :: n1, n2, n3, a1, a2, a3, niter, nsc, na, i, linflt, filliter, dim
  integer, external                    :: getch
  integer, dimension (:), pointer      :: sc, a, ndat, patch, fakepatch,tmppch
  integer                              :: tt(1000)
  real, dimension (:), pointer         :: data, k0, killdata, dummask
  integer, dimension (:), allocatable  :: center, gap, psize, pos, cartflt
  real, dimension (:), pointer         :: ddat, odat, wei
  real                                 :: d1, d2, d3, o1, o2, o3, eps
  logical                              :: rad, ritestat, stat, lmask(:)
  type(nfilter)                        :: aa
  character(len=1024)                  :: out

  call sep_init(SOURCE)
  dim = sep_dimension()
! Read history of data
  call from_history( "n1", n1);       call from_history( "n2", n2);       call from_history( "n3", n3)
  call from_history( "d1", d1, 1.0);  call from_history( "d2", d2, 1.0);  call from_history( "d3", d3, 1.0)
  call from_history( "o1", o1, 0.0);  call from_history( "o2", o2, 0.0);  call from_history( "o3", o3, 0.0)
  call to_history( "n", (/n1,n2,n3/) )
! Read parameters
  call from_param ("niter", niter, 50)
  call from_param ("rad",rad,.false.)
  call from_param ("eps",eps,0.1)
  call from_param ("out",out,'npeffy.H')
! Read scale parameters
  nsc = getch("sc","d",tt)
  allocate( sc(nsc) ); sc(1:nsc)=tt(1:nsc)
  call from_param ("sc", sc)
! allocate data arrays
  allocate(ndat(dim), a(dim), ddat(dim), odat(dim), center(dim), gap(dim), patch(dim), fakepatch(dim) )
  ndat = (/n1,n2,n3/); ddat = (/d1,d2,d3/); odat = (/o1,o2,o3/)
  allocate( data(product(ndat)), k0(product(ndat)), tmppch(product(ndat)), wei(product(ndat)) )

! Read mask of known data
  call sep_read( data )
  k0 = 1.
  if (exist_file ("maskin"))  call sep_read( k0, "maskin")

! read filter parameters
  call from_param( "a", a )
  center=1;  where( a(2:) > 1) center (:dim-1) = a (:dim-1)/2+1
  call from_param ("center", center, center)
  gap = 0
  call from_param ("gap", gap, gap)
  patch = 1
  call from_param ("patch", patch, patch)
  fakepatch = product(patch) ! total number of micropatches

! allocate helix
write(0,*)"before createhelix ndat", ndat
write(0,*)"center", center
write(0,*)"gap", gap
write(0,*)"a", a
write(0,*)"fp", fakepatch
  aa = createnhelix( ndat, center, gap, a, fakepatch)
write(0,*)"after createhelix"

! create patch table for filter
  allocate(aa%pch(product(ndat)), psize(dim), pos(dim), cartflt(dim))
  psize = ndat/patch
  do i=1,dim
     if (mod(ndat(i),patch(i))>0) then
        write(0,*)"patch choice poor",mod(patch(i),ndat(i))
     end if
  end do
  do i = 1,product(ndat)
     call line2cart(ndat,i,pos)
     cartflt = (pos-1)/psize +1
     call cart2line(patch,cartflt,linflt)
     tmppch(i) = linflt
  end do

  aa%pch = tmppch
! bound filter
write(0,*)"before nbound"
  call nbound_bob( ndat, aa)
write(0,*)"after nbound"
! call main program
  call nmultipef3_find( data, aa, sc, ndat, ddat, odat, a, niter, k0, patch, eps )

! use PEF to fill data
  allocate(lmask(size(k0)))
  lmask=.false.
  where(k0/=0) lmask=.true.
  wei = 1.
  call nbound_bob( ndat, aa)

  ritestat=rite_nfilt(out,aa,ndat,odat,ddat)
!  call nsfill_data(filliter, data, aa, lmask, .false., wei )
  write(0,*)"filter written?",ritestat
! finish up
!  call sep_write (data)
  call sep_close ()
  deallocate( data)
  deallocate( k0)
  call ndeallocate(aa)
  deallocate (ndat, center, gap, a, ddat, odat, patch, psize, pos, cartflt, tmppch) 
  call exit (0)
end program NMultipef3

































































